/**
 * fluent-adflow-widget.js
 *
 * Licensed under the Fluent Software Development Kit (SDK) Terms of Use
 * Version 1.0 (the "License");
 *
 * You may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at https://fluentco.com
 */

import {
  requireNativeComponent,
  NativeEventEmitter,
  NativeModules,
  type HostComponent,
} from 'react-native';
import React, { Component } from 'react';
import packageJson from '../package.json';

const eventManagerEmitter = new NativeEventEmitter(
  NativeModules.AdsViewEventEmitter
);
const packageName = packageJson.name;
const packageVersion = packageJson.version;

type FluentAdFlowWidgetProps = {
  style?: any;
  params: any;
  onAdShow?: any;
  wrapperName?: any;
  wrapperVersion?: any;
};
export interface FluentAdFlowWidgetState {
  height: number;
  viewId: String;
}

export class FluentAdFlowWidgetView extends Component<
  FluentAdFlowWidgetProps,
  FluentAdFlowWidgetState
> {
  subscriberOnAdFlowContainerClosed = eventManagerEmitter.addListener(
    'onAdFlowContainerClosed',
    (viewId) => {
      if (viewId == this.state.viewId) {
        this.props.onAdShow && this.props.onAdShow(false);
        console.log('>>>>>> onAdFlowContainerClosed -> viewId: ', viewId);
        this.setState({ height: 0 });
      }
    }
  );

  subscriberOnAdFlowContainerShow = eventManagerEmitter.addListener(
    'onAdFlowContainerShow',
    (viewId) => {
      if (viewId == this.state.viewId) {
        console.log('>>>>>> onAdFlowContainerShow -> viewId: ', viewId);
        this.props.onAdShow && this.props.onAdShow(true);
        this.setState({ height: this.props.style.height });
      }
    }
  );

  subscriberOnAdFlowContainerHeightChanged = eventManagerEmitter.addListener(
    'onAdFlowContainerHeightChanged',
    (result) => {
      const resArry = result.split('$');
      if (resArry.length > 1) {
        const viewId = resArry[0];
        const height = resArry[1];
        if (viewId == this.state.viewId) {
          console.log('>>>>>> onAdFlowContainerHeightChanged -> viewId: ', viewId);
          console.log('                                      -> height : ', parseInt(height));
          this.setState({ height: parseInt(height) });
        }
      }
    }
  );

  subscriberOnError = eventManagerEmitter.addListener('onError', (viewId) => {
    if (viewId == this.state.viewId) {
      console.log('>>>>>> onError -> viewId: ', viewId);
      this.setState({ height: 0 });
    }
  });

  subscriberOnValidationFailed = eventManagerEmitter.addListener(
    'onValidationFailed',
    (viewId) => {
      if (viewId == this.state.viewId) {
        console.log('>>>>>> onValidationFailed -> viewId: ', viewId);
      }
    }
  );

  subscriberOnNotified = eventManagerEmitter.addListener(
    'onNotified',
    (viewId) => {
      if (viewId == this.state.viewId) {
        console.log('>>>>>> onNotified -> viewId: ', viewId);
      }
    }
  );

  constructor(props: FluentAdFlowWidgetProps) {
    super(props);
    this.state = {
      height: 0,
      viewId: Date.now() + '' + Math.floor(Math.random() * 1000),
    };
  }

  override render() {
    var params = { ...this.props.params };
    params.viewId = this.state.viewId;
    return (
      <WidgetNativeComponent
        style={[this.props.style, { height: this.state.height }]}
        params={params}
        wrapperName={packageName}
        wrapperVersion={packageVersion}
      />
    );
  }

  override componentWillUnmount() {
    this.subscriberOnAdFlowContainerClosed.remove();
    this.subscriberOnAdFlowContainerShow.remove();
    this.subscriberOnAdFlowContainerHeightChanged.remove();
    this.subscriberOnError.remove();
    this.subscriberOnValidationFailed.remove();
    this.subscriberOnNotified.remove();
  }
}

export const WidgetNativeComponent: HostComponent<FluentAdFlowWidgetProps> =
  requireNativeComponent('FluentAdFlowWidgetView');

export default FluentAdFlowWidgetView;
